@@ -28,6 +28,7 @@ from typing import Optional
 from typing import List
 from typing import Iterable
 from typing import Any
+from typing import Set
 
 import attr
 import parso
@@ -230,22 +231,19 @@ async def sync_map(
         task_status=trio.TASK_STATUS_IGNORED,
     ) -> None:
 
+        await prev_done.wait()
         async with limiter:
-            # XXX Is this relying on a Trio implementation detail to retain the order?
-            task_status.started()
             result = await function(item)
 
-        await prev_done.wait()
         await send_result.send(result)
         self_done.set()
 
-
     async def consume_input(nursery) -> None:
         prev_done = trio.Event()
         prev_done.set()
         async for item in iterable:
             self_done = trio.Event()
-            await nursery.start(wrapper, prev_done, self_done, item, name=function)
+            nursery.start_soon(wrapper, prev_done, self_done, item, name=function)
             prev_done = self_done
         await prev_done.wait()
         await send_result.aclose()
@@ -262,7 +260,7 @@ async def async_map_unordered(
 ) -> AsyncIterator[AsyncIterable[U]]:
     send_result, receive_result = trio.open_memory_channel[U](0)
     limiter = trio.CapacityLimiter(max_concurrent)
-    remaining_tasks = set()
+    remaining_tasks: Set[int] = set()
 
     async def wrapper(task_id: int, item: T) -> None:
         async with limiter:
@@ -321,6 +319,41 @@ async def async_filter(
         yield receive_result
         nursery.cancel_scope.cancel()
 
+@async_generator.asynccontextmanager
+async def sync_filter(
+    function: Callable[[T], Awaitable[T]], iterable: AsyncIterable[T], max_concurrent
+) -> AsyncIterator[AsyncIterable[T]]:
+    send_result, receive_result = trio.open_memory_channel[T](0)
+
+    limiter = trio.CapacityLimiter(max_concurrent)
+
+    async def wrapper(prev_done: trio.Event, self_done: trio.Event, item: T) -> None:
+
+        await prev_done.wait()
+
+        async with limiter:
+            result = await function(item)
+
+
+        if result:
+            await send_result.send(item)
+        self_done.set()
+
+    async def consume_input(nursery) -> None:
+        prev_done = trio.Event()
+        prev_done.set()
+        async for item in iterable:
+            self_done = trio.Event()
+            nursery.start_soon(wrapper, prev_done, self_done, item, name=function)
+            prev_done = self_done
+        await prev_done.wait()
+        await send_result.aclose()
+
+    async with trio.open_nursery() as nursery:
+        nursery.start_soon(consume_input, nursery)
+        yield receive_result
+        nursery.cancel_scope.cancel()
+
 
 SENTINEL = object()
 
